#pragma offload_attribute(push, target(mic))
#include <algorithm>
#include <cstdlib>
#include <chrono>
#include <iostream>
#include <omp.h>
#include <stdint.h>
#pragma offload_attribute(pop)

#define IMCI_ALIGNMENT 64
#define ARR_SIZE (1024*1024)
#define ITERS_PER_LOOP 128
#define LOOP_COUNT (64*1024*1024)

using namespace std;
using namespace std::chrono;

#pragma omp declare target
float a;
float fa[ARR_SIZE] __attribute__((aligned(IMCI_ALIGNMENT)));
float fb[ARR_SIZE] __attribute__((aligned(IMCI_ALIGNMENT)));

inline void init()	{
	a = 1.1f;
	#pragma omp simd aligned(fa, fb: IMCI_ALIGNMENT)
	for (uint64_t i = 0; i < ARR_SIZE; ++i)	{
		fa[i] = ((float) i) + 0.1f;
		fb[i] = ((float) i) + 0.2f;
	}
}

inline void kernel()	{
	uint64_t offset = omp_get_thread_num()*ITERS_PER_LOOP;
	for (uint64_t j = 0; j < LOOP_COUNT; ++j)	{
		#pragma omp simd aligned(fa,fb: IMCI_ALIGNMENT)
		for (uint64_t k = 0; k < ITERS_PER_LOOP; ++k)
			fa[k+offset] = a*fa[k+offset]+fb[k+offset];
	}
}
#pragma omp end declare target


int main(int argc, char* argv[]) {

	high_resolution_clock::time_point start = high_resolution_clock::now();

	uint64_t numThreads = 1;

	#pragma offload target(mic: 0)
	{
		init();
		numThreads = omp_get_max_threads();
		#pragma omp parallel
		{
			kernel();
		}
	}

	double totalDuration = duration_cast<duration<double>>(high_resolution_clock::now()-start).count();

	double gflops = 1e-9*numThreads*LOOP_COUNT*ITERS_PER_LOOP*2;
	cout<<"GFlops = "<<gflops<<", Secs = "<<totalDuration<<", GFlops per sec = "<<gflops/totalDuration<<endl;

	return 0;
}

